@@ -5,8 +5,9 @@ module Network.HPACK (
   -- * Encoding and decoding
     encodeHeader
   , decodeHeader
-  -- * Encoding in a buffer
-  , encodeHeaderBuffer
+  -- * Encoding and decoding with token
+  , encodeTokenHeader
+  , decodeTokenHeader
   -- * DynamicTable
   , DynamicTable
   , defaultDynamicTableSize
@@ -28,6 +29,12 @@ module Network.HPACK (
   , Header
   , HeaderName
   , HeaderValue
+  , TokenHeaderList
+  , TokenHeader
+  -- * Value table
+  , ValueTable
+  , getHeaderValue
+  , toHeaderTable
   -- * Basic types
   , Size
   , Index
