@@ -1,7 +1,11 @@
 module Network.HPACK.HeaderBlock (
     decodeHeader
+  , decodeTokenHeader
+  , ValueTable
+  , toHeaderTable
+  , getHeaderValue
   , encodeHeader
-  , encodeHeaderBuffer
+  , encodeTokenHeader
   ) where
 
 import Network.HPACK.HeaderBlock.Decode
