@@ -4,7 +4,9 @@ module Network.HPACK.Huffman (
   , HuffmanDecoding
   -- * Encoding/decoding
   , encode
+  , encodeHuffman
   , decode
+  , decodeHuffman
   ) where
 
 import Network.HPACK.Huffman.Decode
