@@ -11,10 +11,10 @@ module Network.HPACK.Table.Entry (
   -- * Header and Entry
   , toEntry
   , toEntryToken
-  , fromEntry
   -- * Getters
   , entrySize
-  , entryHeader
+  , entryTokenHeader
+  , entryToken
   , entryHeaderName
   , entryHeaderValue
   -- * For initialization
@@ -23,7 +23,7 @@ module Network.HPACK.Table.Entry (
   ) where
 
 import qualified Data.ByteString as BS
-import Network.HPACK.Table.Token
+import Network.HPACK.Token
 import Network.HPACK.Types
 
 ----------------------------------------------------------------
@@ -32,7 +32,7 @@ import Network.HPACK.Types
 type Size = Int
 
 -- | Type for table entry. Size includes the 32 bytes magic number.
-data Entry = Entry Size Token Header deriving Show
+data Entry = Entry Size Token HeaderValue deriving Show
 
 ----------------------------------------------------------------
 
@@ -44,23 +44,24 @@ headerSize (k,v) = BS.length k
                  + BS.length v
                  + headerSizeMagicNumber
 
+headerSize' :: Token -> HeaderValue -> Size
+headerSize' t v = BS.length (tokenFoldedKey t)
+                + BS.length v
+                + headerSizeMagicNumber
+
 ----------------------------------------------------------------
 
 -- | From 'Header' to 'Entry'.
 toEntry :: Header -> Entry
-toEntry kv = Entry siz TOTHER kv
+toEntry kv@(k,v) = Entry siz t v
   where
-    !siz = headerSize kv
-
-toEntryToken :: Header -> Entry
-toEntryToken kv@(k,_) = Entry siz t kv
-  where
-    !siz = headerSize kv
     !t = toToken k
+    !siz = headerSize kv
 
--- | From 'Entry' to 'Header'.
-fromEntry :: Entry -> Header
-fromEntry (Entry _ _ kv) = kv
+toEntryToken :: Token -> HeaderValue -> Entry
+toEntryToken t v = Entry siz t v
+  where
+    !siz = headerSize' t v
 
 ----------------------------------------------------------------
 
@@ -68,23 +69,27 @@ fromEntry (Entry _ _ kv) = kv
 entrySize :: Entry -> Size
 entrySize (Entry siz _ _) = siz
 
--- | Getting 'Header'.
-entryHeader :: Entry -> Header
-entryHeader (Entry _ _ kv) = kv
+-- | Getting 'TokenHeader'.
+entryTokenHeader :: Entry -> TokenHeader
+entryTokenHeader (Entry _ t v) = (t, v)
+
+-- | Getting 'Token'.
+entryToken :: Entry -> Token
+entryToken (Entry _ t _) = t
 
 -- | Getting 'HeaderName'.
 entryHeaderName :: Entry -> HeaderName
-entryHeaderName (Entry _ _ (k,_)) = k
+entryHeaderName (Entry _ t _) = tokenFoldedKey t
 
 -- | Getting 'HeaderValue'.
 entryHeaderValue :: Entry -> HeaderValue
-entryHeaderValue (Entry _ _ (_,v)) = v
+entryHeaderValue (Entry _ _ v) = v
 
 ----------------------------------------------------------------
 
 -- | Dummy 'Entry' to initialize a dynamic table.
 dummyEntry :: Entry
-dummyEntry = Entry 0 TOTHER ("dummy","dummy")
+dummyEntry = Entry 0 tokenMax "dummyValue"
 
 -- | How many entries can be stored in a dynamic table?
 maxNumbers :: Size -> Int
