@@ -7,6 +7,8 @@ module Network.HPACK.Types (
   , HeaderStuff
   , Header
   , HeaderList
+  , TokenHeader
+  , TokenHeaderList
   -- * Misc
   , Index
   , HIndex(..)
@@ -26,6 +28,7 @@ import Data.ByteString (ByteString)
 import Data.Typeable
 import Data.Word (Word8)
 import Foreign.Ptr (Ptr)
+import Network.HPACK.Token (Token)
 
 ----------------------------------------------------------------
 
@@ -44,6 +47,10 @@ type HeaderList = [Header]
 -- | To be a 'HeaderName' or 'HeaderValue'.
 type HeaderStuff = ByteString
 
+type TokenHeader = (Token, HeaderValue)
+
+type TokenHeaderList = [TokenHeader]
+
 ----------------------------------------------------------------
 
 -- | Index for table.
@@ -89,6 +96,7 @@ data DecodeError = IndexOverrun Index -- ^ Index is out of range
                  | TooLargeTableSize -- ^ A peer tried to change the dynamic table size over the limit
                  | IllegalTableSizeUpdate -- ^ Table size update at the non-beginning
                  | HeaderBlockTruncated
+                 | IllegalHeaderName
                  deriving (Eq,Show,Typeable)
 
 instance Exception DecodeError
